#MeMo by Fabiola Murguia-Flores, et al., 2017.
#University of Bristol
#fm13804@bristol.ac.uk
#Script to run soil MEthane uptake MOdel (MeMo) on a monthly resolution for the year 2000 (example). Model can be modified to run other periods, if needed please contact me. 

#First step is to replace all PATH, to the correct path to the data and output in your computer

#clear workspace and set memory size
memory.size(100000)
rm(list = ls())

#libraries
library(ncdf4)

#constant
Fc=586.7*30          #unit conversion 

#Time Steps (number of months)
tms<-12
																			
#DRIVING DATA
a<-nc_open("PATH/2000_driving_data.nc")

#Atmospheric methane concentration
#ch<-read.csv("PATH/ch4.txt",h=T) #Path to global atmospheric methane concentration, must be in ppb. Only if other data is used
ch<-c(1776.14722,1776.53442,1776.88123,1775.48779,1773.28687,1771.81567,1769.44678,1769.38977,1771.9812,1774.64258,1775.63916,1774.76831) #data for year 2000
lat<-1:180
#ATM mixing ration of CH4
mixing<-lat*0.00045+0.96
mix<-t(replicate(360,mixing))
CH4mix<-array(dim=c(360,180,tms),0)
for(j in 1:tms){
	CH4mix[,,j]<-mix*ch[j]
}

#climate forcing data
#Temperature
#a<-nc_open("PATH/temp.nc") #Path to temperature data (netcdf), must be in degrees C. Only if other data is used.
TE<-ncvar_get(a,"Temperature")[,,1:tms]		
		

#Soil Moisture
#a<-nc_open("PATH/SM.nc") #Path to soil moisture (netcdf), must be in %. Only if other data is used
SM<-ncvar_get(a,"Soil_Moisture")[,,1:tms]
SM<-ifelse(SM>=1,1,SM)
SM<-ifelse(SM<=0,0,SM)	

#Soil data
#Bulk Density
#a<- nc_open("PATH/BulkDensity.nc") #Path to bulk density, must be in g/cm3. Only if other data is used.
BD<-ncvar_get(a,"Bulk_Density")		
BD<-BD
PHI<-(1-(BD/2.65))			#Total pore volume, must be in cm3/cm3
PHI<-ifelse(PHI>1,NaN,PHI)

#Clay Content
#a<- nc_open("PATH/clay.nc") #Path to clay content, must be in % (0-100). Only if other data is used.
clay<-ncvar_get(a,"Clay_Content")
BC<-15.9*clay+2.91

#L maximum uptake depth (for the example already provided L, but it can be used dynamically for each time step inside the model)
Ldep<-ncvar_get(a,"Uptake_Depth")
Ldep<-apply(Ldep,c(1,2),mean) #remove if L is dynamic
LDEP<-replicate(tms,Ldep)  #remove if L is dynamic
	
#Other forcing
#Nitrogen Deposition
#a<-nc_open("PATH/NDEP_regrid.nc")  Nitrogen Deposition data (mg/m2/yr). Only if other data is used. 
depN<-ncvar_get(a,"Nitrogen_deposition")[,,1:tms]
An<-0.4765			     #unit conversion												  
z<-5											
bd<-replicate(tms,BD)							    				
rNH<- depN/bd*z
Inib<- 0.33
rn<-1-(rNH*Inib)*An

#Multiple Ko values
ko= 5*10^-5								  #basal oxidation rate global (Curry et al. 2007)
ktr= 1.57*10^-5                                         #basal oxidaion rate tropics
kte= 4.05*10^-5                                         #basal oxidation rate temperate forest
kgs= 3.57*10^-5                                         #basal oxidation rate savanna

#Vegetation distribution mask
#a<-nc_open("PATH/VEG_1x1.nc") #Path to vegetation mask. Only if other mask is used.
veg<-ncvar_get(a,"Vegetation")
namx<-c("Tropical Evergreen Forest","Tropical Deciduous Forest","Temperate Broadleaf Evergreen Forest","Temperate Needleleaf Evergreen Forest","Temperate Deciduous Forest","Boreal Evergreen Forest","Boreal Deciduous Forest","Mixed Forest","Savanna","	Grassland/Steppe","Dense Shrubland","Open Shrubland","Tundra","Desert","Polar Desert/Rock/Ice")

#Assing Ko for different vegetation types
tro=c(1)
temp=c(2,3,4,5,6,7,8)
gras=c(9,10,11,12)
ot=c(13,14,15)
map1=ifelse(round(veg)<=1,ktr,0)
map11=ifelse(round(veg)==2,ko,0)
map2=ifelse(round(veg)<=5 & round(veg)>=3,kte,0)
map21=ifelse(round(veg)<=9 & round(veg)>=6,ko,0)
map3=ifelse(round(veg)==10,ko,0)
map31=ifelse(round(veg)<=12 & round(veg)>=11,kgs,0)
map4=ifelse(round(veg)>=13,ko,0)
map5=map1+map2+map3+map4+map31+map21+map11 #Map of multipple k0


#FLUX CALCULATION
FLUX<-array(0,dim=c(360,180,tms))
for(n in 1:360){
	for(m in 1:180){
		if(is.na(TE[n,m,1])){ #optimization conditions to skip ocean points, proving TE (temp) is only for land.
			FLUX[n,m,]<-NaN
			next
		}else{
		ch4mix<-CH4mix[n,m,]
		t<-TE[n,m,]
		sm<-SM[n,m,]
		RN<-rn[n,m,]
		L<-LDEP[n,m,]
		
		#Oxidation (KD) calculation
		rt=ifelse(t<0,1/exp(-t),exp(0.1515+0.05238*t-5.94*(10^-7)*t^4))				#Temperature modifier
		rsm<-exp(-1*(sm-0.2)^2/(2*0.2^2))/(0.2*sqrt(2*pi))/2         				#Normal fitted soil moisture distribution
		rsm1<-ifelse(rsm>=1,1,rsm)
		rs07<-(1-((log(1/(sm*100))-log(0.2))/(log(100)-log(0.2))))^0.8				#RSM from Curry (2007) distribution for dry soils
		rs07<-rs07/1.18												#Scaling factor or RSM (so rsmax=1)
		rsm7<-ifelse(sm>0.2,1,rs07)
		rsmm15<-ifelse(sm<=0.2,rsm7,rsm1)									#RSM modifier
		rsmm15<-ifelse(sm==1,0,rsmm15)
		rsmm15<-ifelse(sm==0,0,rsmm15)							
		k1=replicate(tms,map5[n,m])
		KD=RN*rsmm15*rt*k1									#OXIDATION RATE BY BACTERIA for any given point (for all timesteps)
		
		#Diffusion of CH4 in the soil
		bc<-replicate(tms,BC[n,m])
		phi<-replicate(tms,PHI[n,m])
		PHIair<-phi*(1-sm)									#Air filled porosity
		do= 0.196											#Diffusivity of CH4 in air at STP cm2/s
		gs= phi^(4/3)*(PHIair/phi)^(1.5+3/bc)						#Soil porosity barrier
		gt= 1+0.0055*t 										#Temperature effect on diffusivity
		DC=do*gs*gt											#DIFFUSION of CH4 in the soil
		
		#Flux calculation
		CH4at<-ch4mix/1000
		A<-CH4at*exp(sqrt(KD/DC)*L)/(-exp(-sqrt(KD/DC)*L)+exp(sqrt(KD/DC)*L))
		B<--CH4at*exp(-sqrt(KD/DC)*L)/(-exp(-sqrt(KD/DC)*L)+exp(sqrt(KD/DC)*L))
		J<--DC*(-A*sqrt(KD/DC)+B*sqrt(KD/DC))*Fc/2			#Equation to calculate flux
		DC<-ifelse(DC==0,0.0000001,DC)
		FLUX[n,m,]<-J										#FINAL FLUX
		}
	}
}

#Mask
mask0<-ncvar_get(a,"mask")
FLUX<-FLUX*replicate(tms,mask0)							#adding a mask if needed (i.e. to mask desert regions)
FLUX2<-ifelse(FLUX>=1e6,NaN,FLUX)

#Monthly to Annual Conversion
fu<-array(FLUX2,dim=c(360,180,12,tms/12))
FLUXan<-apply(fu,c(1,2,4),sum,na.rm=T)
FLUXan<-FLUXan*replicate((tms/12),mask0)

#Save Monthly Files
Lat<-ncdim_def("Lat", "degrees_north", -89.5:89.5, unlim=FALSE, create_dimvar=TRUE, longname="Latitude")
Lon<-ncdim_def("Lon","degrees_east", -179.5:179.5, unlim=FALSE, create_dimvar=TRUE, longname="Longitude")
TIME<-ncdim_def("Time","months", 1:tms, unlim=FALSE, create_dimvar=TRUE, longname="Longitude")
NPP<-ncvar_def("CH4uptake", "gC/m2/month", list(Lon,Lat,TIME), missval=NaN, longname="CH4uptake", prec="float", shuffle=FALSE, compression=NA, chunksizes=NA)
nc<-nc_create("PATH/MeMo_2000_monthly.nc", list(NPP), force_v4=FALSE, verbose=FALSE)
ncvar_put(nc, NPP,FLUX2,start=NA,count=NA,verbose=FALSE)
nc_close(nc)

#Save Annual Files
TIME<-ncdim_def("Time","years", 1:(tms/12), unlim=FALSE, create_dimvar=TRUE, longname="Longitude")
NPP<-ncvar_def("CH4uptake", "gC/m2/year", list(Lon,Lat,TIME), missval=NaN, longname="CH4uptake", prec="float", shuffle=FALSE, compression=NA, chunksizes=NA)
nc<-nc_create("PATH/MeMo_2000_annual.nc", list(NPP), force_v4=FALSE, verbose=FALSE)
ncvar_put(nc, NPP,FLUXan,start=NA,count=NA,verbose=FALSE)
nc_close(nc)




   